#  Clasificatori bazati pe reguli

library(caret)    # R framework for training/testing classification and regression models
library(RWeka)    # R interface to Weka classifiers
library(mlbench)  # datasets from UCI Machine Learning repository

#########################
# Exemplu a):  iris - full data for training - simple illustration of the training process
#########################
data(iris)
TrainData <- iris[,1:4]
TrainClasses <- iris[,5]

jripFit <- train(TrainData, TrainClasses, method = "JRip") # varianta a algoritmului RIPPER
jripFit$finalModel

oneRFit <- train(TrainData, TrainClasses, method = "OneR")
oneRFit$finalModel

###########################
# b) Breast Cancer
###########################

#data(BreastCancer)
date <- read.arff("datasets/breast-cancer.arff")

# eliminarea inregistrarilor cu valori lipsa 
date <- na.omit(date)

dataSet <- date[,1:9]
dataClass <- date[,10]

# partitionare set de date: training (75%) and testing (25%) 
# dataClass = the Class attribute
inTrain <- createDataPartition(dataClass, p = 0.75, list = FALSE)

trainingSet <- dataSet[inTrain,]
testingSet <- dataSet[-inTrain,]

trainingClass <- dataClass[inTrain]
testingClass <- dataClass[-inTrain]

# validare bazata pe bootstrapping
bootstrappingControl <- trainControl(number = 100) # bootstrapping based on 100 resamples (default value:25)

# J RIPPER (Weka version)
jripFit <- train(trainingSet, trainingClass, method = "JRip", trControl=bootstrappingControl)
jripFit$finalModel 

resTesting <- predict(jripFit, testingSet)
confusionMatrix(data = resTesting, reference = testingClass)

# One R (Weka version)
OneRFit <- train(trainingSet, trainingClass, method = "OneR", trControl=bootstrappingControl)
OneRFit$finalModel

resTesting1R <- predict(OneRFit, testingSet)
confusionMatrix(data = resTesting1R, reference = testingClass)

###########################
# c) Titanic
trainTitanic <- read.csv("datasets/trainTitanic.csv")
testTitanic <- read.csv("datasets/testTitanic.csv")

# Exercitiu:  
# (i) pregatiti seturile de antrenare si testare
# (ii) antrenati un model de tip JRip/OneR 
# (iii) analizati performanta modelului 





