library(caret)
library(RWeka)
data(BreastCancer)

date <- read.arff("datasets/breast-cancer.arff")

# eliminarea inregistrarilor cu valori lipsa - nu e neaparat necesara pt arbori de decizie
#BreastCancer <- na.omit(BreastCancer)

dataSet <- date[,1:9]
dataClass <- date[,10]

# transformare in valori numerice - nu e necesara pentru arborii de decizie
#for (i in 1:(dim(dataSet)[2]))
#  dataSet[,i] <- as.numeric(dataSet[,i])

# partitionare in set antrenare (75%) si set testare (25%) 
# dataClass = the Class attribute
inTrain <- createDataPartition(dataClass, p = 0.75, list = FALSE)

# separare dataSet in trainingSet si testingSet (separat atribute predictive si atribut de clasa)
trainingSet <- dataSet[inTrain,]
testingSet <- dataSet[-inTrain,]

trainingClass <- dataClass[inTrain]
testingClass <- dataClass[-inTrain]

# separare DataSet in training si testing (impreuna atribute predictive si atribut de clasa)
trainingDate <- date[inTrain,]
testingDate <- date[-inTrain,]

# construire model
bootstrappingControl <- trainControl(number = 100) # bootstrapping cu 100 selectii (default value:25)

# algoritm rpart
rpartFit <- train(trainingSet, trainingClass, method = "rpart", trControl = bootstrappingControl)
rpartFit2 <- train(Class ~ ., trainingDate, method = "rpart", trControl = bootstrappingControl)

rpartFit$finalModel
plot(rpartFit)
resTesting <- predict(rpartFit, testingSet)
confusionMatrix(data = resTesting, reference = testingClass)


# algoritm J48  (din Weka)
j48Fit <- train(trainingSet, trainingClass, method = "J48",  trControl = bootstrappingControl)
j48Fit$finalModel
plot(j48Fit)
resTesting <- predict(j48Fit, testingSet)
confusionMatrix(data = resTesting, reference = testingClass)


# algorithm C5.0 - pt detalii https://www.rulequest.com/see5-unix.html
c50Fit <- train(trainingSet, trainingClass, method = "C5.0",  trControl = bootstrappingControl)
c50Fit$finalModel
plot(c50Fit)
resTesting <- predict(c50Fit, testingSet)
confusionMatrix(data = resTesting, reference = testingClass)



