# Arbori de decizie / Decision trees - packages "rpart", "party"
# install.packages("party")
library(party)   # pachet party
library(rpart)   # pachet rpart (recursive partitioning)


################################
# Exemplu 1:  iris dataset
################################

data(iris)
dataSet <- iris

# partitionare set de date: antrenare si testare - selectie aleatoare
ind <- sample(2, nrow(dataSet), replace=TRUE, prob=c(0.7, 0.3))
trainData <- iris[ind==1,]
testData <- iris[ind==2,]

# utilizare pachet "party"
# specificarea modelului (toate atributele sunt considerate predictori)
modelParty <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width 

# construirea modelului
treeParty <- ctree(modelParty, data=trainData)

# verificare predictie
table(predict(treeParty), trainData$Species)   # matrice de confuzie
print(treeParty)		# vizualizare arbore (text)
plot(treeParty)         # vizualizare arbore (grafic)


#####################################

# utilizare pachet "rpart" 

# structura model (toate atributele de intrare sunt considerate predictori)
modelRpart <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width

# construire model 
treeRpart <- rpart(modelRpart, data = trainData, control = rpart.control(minsplit = 10))

attributes(treeRpart)
print(treeRpart$cptable)
print(treeRpart)

# grafic arbore
dev.new()  # crearea unei noi ferestre grafice
plot(treeRpart)
text(treeRpart, use.n=TRUE)

#####################################
# Exemplu 2:  Titanic dataset
#####################################

trainTitanic <- read.csv("datasets/trainTitanic.csv")
testTitanic <- read.csv("datasets/testTitanic.csv")

modelTitanicRpart <- Survived ~ Pclass + Sex + Age + SibSp + Parch
treeRpartTitanic <- rpart(modelTitanicRpart, data = trainTitanic, control = rpart.control(minsplit = 10), method="class")

dev.new()
plot(treeRpartTitanic)
text(treeRpartTitanic)

print(treeRpartTitanic)
printcp(treeRpartTitanic)

dev.new()  # utilizare pachet party pt vizualizare arbore construit folosind rpart 
library(partykit)
treeTitanicParty<-as.party(treeRpartTitanic)
plot(treeTitanicParty)

###################################
# Exercitiu:  sa se construiasca un arbore de clasificare pentru BreastCancer dataset
###################################

