# Feedforward Neural Networks (Multilayer Perceptron)

library(caret)	# ClAssification and REgression Training
library(pROC)	# functions for ROC curve
library(mlbench)  # datasets from UCI Machine Learning

## Exercitiu:  (a) neural network-based classification of Breast Cancer data. 
# Variante: nominal vs numeric => retelele neuronale necesita valori numerice 
# analiza influentei standardizarii

data(BreastCancer)

# eliminare inregistrari cu valori lipsa
BreastCancer <- na.omit(BreastCancer)

dataSet <- BreastCancer[,2:10]
dataClass <- BreastCancer[,11]

# transformare atribute in numerice   

for (i in 1:(dim(dataSet)[2]))
  dataSet[,i] <- as.numeric(dataSet[,i])

# partitionare set in training (75%) si testing (25%) 

inTrain <- createDataPartition(dataClass, p = 0.75, list = FALSE)

trainingSet <- dataSet[inTrain,]
testingSet <- dataSet[-inTrain,]

trainingClass <- dataClass[inTrain]
testingClass <- dataClass[-inTrain]


# conversie clasa nominal in numeric {0,1}
# trainingClass <- as.numeric(dataClass[inTrain])-1
# testingClass <- as.numeric(dataClass[-inTrain])-1

# cross validation with 10 folds
cvControl <- trainControl(method = 'cv', number = 10, classProbs = TRUE, verboseIter = TRUE) 

# Antrenare retea neuronala - se incearca diferite valori pentru numarul de unitati ascunse (4,8,10) 
# si pentru coeficientul de regularizare (0,0.1,0.5) = decay factor  
nnFit <- train(trainingSet, trainingClass, method="nnet",  trControl=cvControl, preProcess=c("center","scale"), tuneGrid=expand.grid(size=c(4,8,10), decay=c(0,0.1,0.5)))

# ??? Care este cel mai bun nr de unitati ascunse?  (Indicatie:  se utilizeaza nnFit$finalModel)

nnFit$finalModel

# vizualizare acuratete pt diferite combinatii ale hiperparametrilor - specificati in tuneGrid
plot(nnFit)

# testare
resTesting <- predict(nnFit, testingSet, type="prob")

# converting the neural network outputs in class names (as factors)

# Varianta 1
convertProbNominal <- function(nnResult,levelNames){
  n <- dim(nnResult)[1]
  nnResultNominal <- rep("",n)
  for (i in 1:n){
    if (nnResult[i,1]>nnResult[i,2])  # select the class having a higher probability
      nnResultNominal[i] <- levelNames[1]  
    else
      nnResultNominal[i] <- levelNames[2]
  }
  return(as.factor(nnResultNominal))
}

resTestingNominal <- convertProbNominal(resTesting,c("benign","malignant"))

# Varianta 2
resTestingNominal <- predict(nnFit, testingSet, type="raw")


# ??? comparati rezultatul retelei (resTestingNominal) cu etichetele corecte (testingClass)

confusionMatrix(data = resTestingNominal, reference = testingClass)

nnROC <- roc(testingClass,resTesting[,"malignant"]) 
plot(nnROC, type="S", print.thres=c(0.1,0.25,0.5,0.75,0.9))

# ??? Tema: proiectati o retea neuronala pt Titanic dataset / DNA (bioinfo)
